/*
 * JFlex spec for a simple calculator
 */

// section one ... user code that is copied verbatim into the top of the generated
// java file

%%

// section 2 ... options and declare regex constants
%class Lexer        // Name of the class to produce from this spec
%unicode            // supported character set to be scanned
%line               // turn on line number tracking
%column             // turn on column number tracking
%type Token         // specify the return type of tokens

%{
    // code here gets copied into the top of the Lexer class
    private Token token(Tok type) {
        return new Token(type, yyline+1, yycolumn+1);
    }

    private Token token(Tok type, Object value) {
        return new Token(type, yyline+1, yycolumn+1, value);
    }
%}

// macro/constant regex for different token types
Identifier = [a-z][_A-Za-z0-9]*
IntegerLiteral = [0-9]+
FloatLiteral = [0-9]+"." | "."[0-9]+ | [0-9]+"."[0-9]+
Whitespace = [ \t\n]

%%

// section three ... lexical analysis rules
<YYINITIAL> {
    "+"             { return token(Tok.PLUS); }
    "-"             { return token(Tok.MINUS); }

    // yytext() returns the matched lexeme
    {Identifier}     { return token(Tok.IDENT, yytext()); }
    {IntegerLiteral} { return token(Tok.INT, yytext()); }

    {Whitespace}     { /* ignore */ }
}

// Catch all pattern for if we don't match
[^]                 { throw new Error(yyline + ":" + yycolumn +
                        " could not match <" + yytext() + ">"); }


